import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:syncrow_web/pages/common/custom_web_textfield.dart';
import 'package:syncrow_web/pages/common/date_time_widget.dart';
import 'package:syncrow_web/pages/common/default_button.dart';
import 'package:syncrow_web/pages/common/info_dialog.dart';
import 'package:syncrow_web/pages/visitor_password/bloc/visitor_password_bloc.dart';
import 'package:syncrow_web/pages/visitor_password/bloc/visitor_password_event.dart';
import 'package:syncrow_web/pages/visitor_password/bloc/visitor_password_state.dart';
import 'package:syncrow_web/pages/visitor_password/view/add_device_dialog.dart';
import 'package:syncrow_web/pages/visitor_password/view/repeat_widget.dart';
import 'package:syncrow_web/utils/color_manager.dart';
import 'package:syncrow_web/utils/constants/assets.dart';
import 'package:syncrow_web/utils/style.dart';

class VisitorPasswordDialog extends StatelessWidget {
  const VisitorPasswordDialog({super.key});

  @override
  Widget build(BuildContext context) {
    Size size = MediaQuery.of(context).size;
    return BlocProvider(
      create: (context) => VisitorPasswordBloc(),
      child: BlocBuilder<VisitorPasswordBloc, VisitorPasswordState>(
        builder: (BuildContext context, VisitorPasswordState state) {
          final visitorBloc = BlocProvider.of<VisitorPasswordBloc>(context);
          bool isRepeat =
              state is IsRepeatState ? state.repeat : visitorBloc.repeat;
          return AlertDialog(
            backgroundColor: Colors.white,
            title: Text(
              'Create visitor password',
              style: Theme.of(context).textTheme.headlineLarge!.copyWith(
                  fontWeight: FontWeight.w400,
                  fontSize: 24,
                  color: Colors.black),
            ),
            content: SingleChildScrollView(
              child: Form(
                key: visitorBloc.forgetFormKey,
                child: Padding(
                  padding: const EdgeInsets.all(5.0),
                  child: ListBody(
                    children: <Widget>[
                      Container(
                        child: Row(
                          children: [
                            Expanded(
                              flex: 2,
                              child: CustomWebTextField(
                                validator: visitorBloc.validate,
                                controller: visitorBloc.userNameController,
                                isRequired: true,
                                textFieldName: 'Name',
                                description: '',
                              ),
                            ),
                            const Spacer(),
                            Expanded(
                              flex: 2,
                              child: CustomWebTextField(
                                validator: visitorBloc.validateEmail,
                                controller: visitorBloc.emailController,
                                isRequired: true,
                                textFieldName: 'Email Address',
                                description:
                                    'The password will be sent to the visitor’s email address.',
                              ),
                            ),
                            const Spacer(),
                          ],
                        ),
                      ),
                      const SizedBox(
                        height: 15,
                      ),
                      Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: [
                          Row(
                            children: [
                              Text(
                                '* ',
                                style: Theme.of(context)
                                    .textTheme
                                    .bodyMedium!
                                    .copyWith(color: Colors.red),
                              ),
                               Text('Access Type',
                                 style: Theme.of(context).textTheme.bodySmall!.copyWith(
                                  color: Colors.black,fontSize: 13),),
                            ],
                          ),
                          Row(
                            children: <Widget>[
                              SizedBox(
                                width: size.width * 0.15,
                                child: RadioListTile<String>(
                                  title:  Text('Online Password',
                                    style: Theme.of(context).textTheme.bodySmall!.copyWith(
                                      color: Colors.black,fontSize: 13),
                                  ),
                                  value: 'Online Password',
                                  groupValue: (state is PasswordTypeSelected)
                                      ? state.selectedType
                                      : visitorBloc.accessTypeSelected,
                                  onChanged: (String? value) {
                                    if (value != null) {
                                      print(value);
                                      context
                                          .read<VisitorPasswordBloc>()
                                          .add(SelectPasswordType(value));
                                    }
                                  },
                                ),
                              ),
                              SizedBox(
                                width: size.width * 0.15,
                                child: RadioListTile<String>(
                                  title:  Text('Offline Password',
                                    style: Theme.of(context).textTheme.bodySmall!.copyWith(
                                      color: Colors.black,fontSize: 13),),
                                  value: 'Offline Password',
                                  groupValue: (state is PasswordTypeSelected)
                                      ? state.selectedType
                                      : visitorBloc.accessTypeSelected,
                                  onChanged: (String? value) {
                                    if (value != null) {
                                      print(value);

                                      context
                                          .read<VisitorPasswordBloc>()
                                          .add(SelectPasswordType(value));
                                    }
                                  },
                                ),
                              ),
                              SizedBox(
                                width: size.width * 0.15,
                                child: RadioListTile<String>(
                                  title:  Text('Dynamic Password',
                                    style: Theme.of(context).textTheme.bodySmall!.copyWith(
                                      color: Colors.black,fontSize: 13),),
                                  value: 'Dynamic Password',
                                  groupValue: (state is PasswordTypeSelected)
                                      ? state.selectedType
                                      : visitorBloc.accessTypeSelected,
                                  onChanged: (String? value) {
                                    if (value != null) {
                                      context
                                          .read<VisitorPasswordBloc>()
                                          .add(SelectPasswordType(value));
                                      visitorBloc.usageFrequencySelected = '';
                                    }
                                  },
                                ),
                              ),
                            ],
                          ),
                           Text(
                              'Only currently online devices can be selected. It is recommended to use when the device network is stable, and the system randomly generates a digital password',
                             style: Theme.of(context).textTheme.bodySmall!.copyWith(
                               fontWeight: FontWeight.w400,
                               color: ColorsManager.grayColor,fontSize: 9),),
                          const SizedBox(
                            height: 20,
                          )
                        ],
                      ),
                      visitorBloc.accessTypeSelected == 'Dynamic Password'
                          ? SizedBox()
                          : Column(
                              crossAxisAlignment: CrossAxisAlignment.start,
                              children: [
                                Row(
                                  children: [
                                    Text(
                                      '* ',
                                      style: Theme.of(context)
                                          .textTheme
                                          .bodyMedium!
                                          .copyWith(color: Colors.red),
                                    ),
                                     Text('Usage Frequency',style: Theme.of(context).textTheme.bodySmall!.copyWith(
                                         color: Colors.black,fontSize: 13),),
                                  ],
                                ),
                                Row(
                                  children: <Widget>[
                                    SizedBox(
                                      width: 200,
                                      child: RadioListTile<String>(
                                        title:  Text('One-Time',style: Theme.of(context).textTheme.bodySmall!.copyWith(
                                            color: Colors.black,fontSize: 13),),
                                        value: 'One-Time',
                                        groupValue:
                                            (state is UsageFrequencySelected)
                                                ? state.selectedFrequency
                                                : visitorBloc
                                                    .usageFrequencySelected,
                                        onChanged: (String? value) {
                                          if (value != null) {
                                            print(value);

                                            context
                                                .read<VisitorPasswordBloc>()
                                                .add(SelectUsageFrequency(
                                                    value));
                                          }
                                        },
                                      ),
                                    ),
                                    SizedBox(
                                      width: 200,
                                      child: RadioListTile<String>(
                                        title:  Text('Periodic',
                                          style: Theme.of(context).textTheme.bodySmall!.copyWith(
                                            color: Colors.black,fontSize: 13),),
                                        value: 'Periodic',
                                        groupValue:
                                            (state is UsageFrequencySelected)
                                                ? state.selectedFrequency
                                                : visitorBloc
                                                    .usageFrequencySelected,
                                        onChanged: (String? value) {
                                          if (value != null) {
                                            context
                                                .read<VisitorPasswordBloc>()
                                                .add(SelectUsageFrequency(
                                                    value));
                                          }
                                        },
                                      ),
                                    ),
                                  ],
                                ),
                                 Text('Within the validity period, each device can be unlocked only once.',
                                   style: Theme.of(context).textTheme.bodySmall!.copyWith(
                                     color: ColorsManager.grayColor,fontSize: 9),)
                              ],
                            ),
                      const SizedBox(
                        height: 20,
                      ),
                      if ((visitorBloc.usageFrequencySelected != 'One-Time' ||
                          visitorBloc.accessTypeSelected != 'Offline Password') &&
                         (visitorBloc.usageFrequencySelected != ''))
                        DateTimeWebWidget(
                          isTime: false,
                          isRequired: true,
                          title: 'Access Period',
                          size: size,
                          endTime: () {
                            visitorBloc.add(SelectTimeVisitorPassword(
                                context: context,
                                isStart: false,
                                isRepeat: false));
                          },
                          startTime: () {
                            visitorBloc.add(SelectTimeVisitorPassword(
                                context: context,
                                isStart: true,
                                isRepeat: false));
                          },
                          firstString: visitorBloc.startTimeAccess.toString(),
                          secondString: visitorBloc.endTimeAccess.toString(),
                        ),
                      const SizedBox(
                        height: 20,
                      ),
                      Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: [
                          Row(
                            children: [
                              Text(
                                '* ',
                                style: Theme.of(context)
                                    .textTheme
                                    .bodyMedium!
                                    .copyWith(color: Colors.red),
                              ),
                               Text('Access Devices',
                                 style: Theme.of(context).textTheme.bodySmall!.copyWith(
                                   color: Colors.black,fontSize: 13),),
                            ],
                          ),
                           Text(
                              'Within the validity period, each device can be unlocked only once.',
                             style: Theme.of(context).textTheme.bodySmall!.copyWith(
                               fontWeight: FontWeight.w400,
                               color: ColorsManager.grayColor,fontSize: 9),),
                          const SizedBox(
                            height: 20,
                          ),
                          if (visitorBloc.usageFrequencySelected == 'Periodic' &&
                              visitorBloc.accessTypeSelected == 'Online Password')
                            SizedBox(
                              width: 100,
                              child: Column(
                                 children: [
                                   Text('Repeat',
                                     style: Theme.of(context).textTheme.bodySmall!.copyWith(
                                         color: Colors.black,fontSize: 13),),
                                   Transform.scale(
                                     scale: .8,
                                     child: CupertinoSwitch(
                                       value: visitorBloc.repeat,
                                       onChanged: (value) {
                                         visitorBloc.add(ToggleRepeatEvent());
                                       },
                                       applyTheme: true,
                                     ),
                                   ),
                                 ],

                              ),
                            ),
                          if (visitorBloc.usageFrequencySelected == 'Periodic' &&
                              visitorBloc.accessTypeSelected == 'Online Password')
                            isRepeat ? const RepeatWidget() : const SizedBox(),
                          Container(
                            decoration: containerDecoration,
                            width: size.width * 0.08,
                            child: DefaultButton(
                              onPressed: () {
                                showDialog(
                                  context: context,
                                  barrierDismissible: false,
                                  builder: (BuildContext context) {
                                    return const AddDeviceDialog();
                                  },
                                );
                              },
                              borderRadius: 8,
                              child: Text('+ Add Device',style: Theme.of(context).textTheme.bodySmall!.copyWith(
                                fontWeight: FontWeight.w400,
                                  color: ColorsManager.whiteColors,fontSize: 12),),
                            ),
                          ),
                        ],
                      ),
                    ],
                  ),
                ),
              ),
            ),
            actionsAlignment: MainAxisAlignment.center,
            actions: <Widget>[
              Container(
                decoration: containerDecoration,
                width: size.width * 0.2,
                child: DefaultButton(
                  borderRadius: 8,
                  onPressed: () {
                    Navigator.of(context).pop(); // Close the dialog
                  },
                  backgroundColor: Colors.white,
                  child: Text(
                    'Cancel',
                      style: Theme.of(context).textTheme.bodySmall!.copyWith(
                      fontWeight: FontWeight.w400,
                      color: ColorsManager.blackColor,fontSize: 16),
                  ),
                ),
              ),
              Container(
                decoration: containerDecoration,
                width: size.width * 0.2,
                child: DefaultButton(
                  onPressed: () {
                    print(selectedDevices);
                    if (visitorBloc.forgetFormKey.currentState!.validate()) {
                      if (selectedDevices.toString() != '[]') {
                        showDialog(
                          context: context,
                          barrierDismissible: false,
                          builder: (BuildContext context) {
                            return InfoDialog(
                              size: size,
                              title: 'Set Password',
                              content:
                                  'This action will update all of the selected\n door locks passwords in the property.\n\nAre you sure you want to continue?',
                              actions: [
                                Container(
                                  decoration: containerDecoration,
                                  width: size.width * 0.1,
                                  child: DefaultButton(
                                    borderRadius: 8,
                                    onPressed: () {
                                      Navigator.of(context)
                                          .pop(); // Close the dialog
                                    },
                                    backgroundColor: Colors.white,
                                    child: Text(
                                      'Cancel',
                                      style: Theme.of(context).textTheme.bodySmall!.copyWith(
                                          fontWeight: FontWeight.w400,
                                          color: ColorsManager.blackColor,fontSize: 16),
                                    ),
                                  ),
                                ),
                                Container(
                                  decoration: containerDecoration,
                                  width: size.width * 0.1,
                                  child: DefaultButton(
                                    borderRadius: 8,
                                    onPressed: () {
                                      if (visitorBloc.usageFrequencySelected == 'One-Time' &&
                                          visitorBloc.accessTypeSelected == 'Online Password') {
                                        visitorBloc.add(
                                            OnlineOneTimePasswordEvent(
                                                context: context,
                                                passwordName: visitorBloc.userNameController.text,
                                                email: visitorBloc.emailController.text));
                                      } else if (visitorBloc.usageFrequencySelected == 'Periodic' &&
                                          visitorBloc.accessTypeSelected == 'Online Password') {
                                        visitorBloc.add(OnlineMultipleTimePasswordEvent(
                                                passwordName: visitorBloc.userNameController.text,
                                                email: visitorBloc.emailController.text,
                                                effectiveTime: visitorBloc.effectiveTimeTimeStamp.toString(),
                                                invalidTime: visitorBloc.expirationTimeTimeStamp.toString()));
                                      } else if (visitorBloc.usageFrequencySelected == 'One-Time' &&
                                          visitorBloc.accessTypeSelected == 'Offline Password') {
                                        visitorBloc.add(OfflineOneTimePasswordEvent(
                                          passwordName: visitorBloc.userNameController.text,
                                          email: visitorBloc.emailController.text,
                                        ));
                                      } else if (visitorBloc.usageFrequencySelected == 'Periodic' &&
                                          visitorBloc.accessTypeSelected == 'Offline Password') {
                                        visitorBloc.add(
                                            OfflineMultipleTimePasswordEvent(
                                                passwordName: visitorBloc.userNameController.text,
                                                email: visitorBloc.emailController.text,
                                                effectiveTime: visitorBloc.effectiveTimeTimeStamp.toString(),
                                                invalidTime: visitorBloc.expirationTimeTimeStamp.toString()));
                                      }
                                    },
                                    child:  Text('Ok',
                                      style: Theme.of(context).textTheme.bodySmall!.copyWith(
                                          fontWeight: FontWeight.w400,
                                          color: ColorsManager.whiteColors,fontSize: 16),
                                    ),
                                  ),
                                ),
                              ],
                            );
                          },
                        );
                      } else {
                        showDialog(
                          context: context,
                          barrierDismissible: false,
                          builder: (BuildContext context) {
                            return AlertDialog(
                              alignment: Alignment.center,
                              content: SizedBox(
                                height: size!.height * 0.15,
                                child: Column(
                                  children: [
                                    SizedBox(
                                      child: SvgPicture.asset(
                                        Assets.deviceNoteIcon,
                                        height: 35,
                                        width: 35,
                                      ),
                                    ),

                                    const SizedBox(
                                      width: 15,
                                    ),
                                    Text(
                                      'Please select devices to continue',
                                      style: Theme.of(context)
                                          .textTheme
                                          .headlineLarge!
                                          .copyWith(
                                          fontSize: 20,
                                          fontWeight: FontWeight.w400,
                                          color: Colors.black),
                                    ),
                                  ],
                                ),
                              ),
                              actionsAlignment: MainAxisAlignment.center,
                              actions: <Widget>[
                                TextButton(
                                  onPressed: () {
                                    Navigator.of(context).pop();
                                  },
                                  child: Text('OK',
                                    style: Theme.of(context).textTheme.bodySmall!.copyWith(
                                      fontWeight: FontWeight.w400,
                                      color: ColorsManager.whiteColors,fontSize: 16),),
                                ),
                              ],
                            );
                          },
                        );
                      }
                    }
                  },
                  borderRadius: 8,
                  child: Text('Ok', style: Theme.of(context).textTheme.bodySmall!.copyWith(
                      fontWeight: FontWeight.w400,
                      color: ColorsManager.whiteColors,fontSize: 16),),
                ),
              ),
            ],
          );
        },
      ),
    );
  }
}
